% Reproduction of performance curves appearing in
% Y. Kopsinis, K. Slavakis, S. Theodoridis, "Online Sparse System Identification and Signal Reconstruction Using
% Projections Onto Weighted $ell_{1}$ Balls," Signal Processing, IEEE Transactions on , vol.59, no.3, pp.936-952, March 2011


%% Reproduction of Fig. 3a. (3b-3c can be similarly obtained)
clear all
L=100;   % System length
S=5;    % No-zero values

N=500; % Number of data (observations)

aver=100;
er1=zeros(1,N);
er2=zeros(1,N);
er3=zeros(1,N);
er4=zeros(1,N);

 noise_variance=0.1; %noise variance
optFig3a_1 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',S,'q',5,'weight',[0.001,0.2],'verbose',0,'output',2);
optFig3a_2 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',S,'q',25,'weight',[0.001,0.2],'verbose',0,'output',2);
optFig3a_3 = SPALOPTIONS(@APL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'q',5,'verbose',0,'output',2);
optFig3a_4 = SPALOPTIONS(@APL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'q',25,'verbose',0,'output',2);

for rep=1:aver
    if mod(rep,10)==0
        fprintf(' %d',rep)
    end
    
    h=[randn(1,S) zeros(1,L-S)]';
    h=h(randperm(L));
    
    x=randn(1,N+L-1);
    y=conv(h,x);
    y=y(L:end-L+1)';
    
    noise_variance=0.1; %noise variance
    y=y+randn(N,1)*sqrt(noise_variance);
    
   
    A=repmat(h,1,N);
    
    [out]= spal(y,x,L,optFig3a_1);
    er1=er1+sum((A-out).^2,1);
    
    [out]= spal(y,x,L,optFig3a_2);
    er2=er2+sum((A-out).^2,1);

    [out]= spal(y,x,L,optFig3a_3,'delta',norm(h,1));
    er3=er3+sum((A-out).^2,1);
    
    [out]= spal(y,x,L,optFig3a_4,'delta',norm(h,1));
    er4=er4+sum((A-out).^2,1);
    
end

fprintf('\n')

figure;hold on
h=plot(1:N,log10(er1/aver/L),'b',1:N,log10(er2/aver/L),'r',1:N,log10(er3/aver/L),'g',1:N,log10(er4/aver/L),'k');
legend(h,{'APWL1, q=5','APWL1, q=25','APL1, q=5','APL1, q=25'})


%% Reproduction of Fig. 5
% A minor difference compared to Fig. 5 is that $\epsilon^{'}_n$ is set
% fixed to 0.05. In fact this simplification gives slightly better results
% than those presented in the paper.

clear all
L=100;
N=1000;

% Number of ensemble averages
aver=100;
er=zeros(1,N);
 noise_variance=0.1; %noise variance
optFig5 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',9,'q',25,'weighteps',0.05,'output',2,'verbose',0);

for rep=1:aver
    if mod(rep,10)==0
        fprintf(' %d',rep)
    end
    
    % generation of the specific time-varying system
    hc{1}=[randn(1,5) zeros(1,L-5)]';
    hc{2}=zeros(L,1);
    hc{2}(1:2:15)=randn(1,8);
    h=zeros(L,N);
    h(:,1:floor(N/2))=repmat(hc{1},1,floor(N/2));
    h(:,floor(N/2)+1:N)=repmat(hc{2},1,N-floor(N/2));
    
    x=randn(1,N+L-1);
    A=convmtx(x,L);
    A=A(:,L:end-L+1);
    y=sum(A.*h,1)';
    
    y=y+randn(N,1)*sqrt(noise_variance);
    
    
    [out]= spal(y,x,L,optFig5);
    
    
    er=er+sum((h-out).^2,1);
    
end

figure
plot(1:N,log10(er/aver/L))


%% Reproduction of Fig. 7. 
clear all
L=100;   % System length
S=5;    % No-zero values

N=500; % Number of data (observations)

aver=10;
er1=zeros(1,N);
er2=zeros(1,N);
er3=zeros(1,N);
er4=zeros(1,N);

 noise_variance=0.1; %noise variance
optFig3a_1 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',S-S*10/100,'q',25,'weight',[0.001,1],'verbose',0,'output',2);
optFig3a_2 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',S,'q',25,'weight',[0.001,1],'verbose',0,'output',2);
optFig3a_3 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',S+S*50/100,'q',25,'weight',[0.001,1],'verbose',0,'output',2);
optFig3a_4 = SPALOPTIONS(@APWL1,'mu',1,'Sla',1.3*sqrt(noise_variance),'delta',2*S,'q',25,'weight',[0.001,1],'verbose',0,'output',2);

for rep=1:aver
    if mod(rep,10)==0
        fprintf(' %d',rep)
    end
    
    h=[randn(1,S) zeros(1,L-S)]';
    h=h(randperm(L));
    
    x=randn(1,N+L-1);
    y=conv(h,x);
    y=y(L:end-L+1)';
    
    y=y+randn(N,1)*sqrt(noise_variance);
    
   
    A=repmat(h,1,N);
    
    [out]= spal(y,x,L,optFig3a_1);
    er1=er1+sum((A-out).^2,1);
    
    [out]= spal(y,x,L,optFig3a_2);
    er2=er2+sum((A-out).^2,1);

    [out]= spal(y,x,L,optFig3a_3);
    er3=er3+sum((A-out).^2,1);
    
    [out]= spal(y,x,L,optFig3a_4);
    er4=er4+sum((A-out).^2,1);
    
end

fprintf('\n')

figure;hold on
h=plot(1:N,log10(er1/aver/L),'b',1:N,log10(er2/aver/L),'r',1:N,log10(er3/aver/L),'g',1:N,log10(er4/aver/L),'k');
legend(h,{'APWL1,-10%','APWL1, 0%','APL1, +50%','APL1, +100%'})
